Index: buffer.h
===================================================================
RCS file: /usr/local/cvsroot/vorbis-tools/ogg123/buffer.h,v
retrieving revision 1.1
diff -u -r1.1 buffer.h
--- buffer.h	2000/12/30 05:44:31	1.1
+++ buffer.h	2001/03/03 21:03:55
@@ -15,18 +15,22 @@
 {
   char status;       /* Status. See STAT_* below. */
   int fds[2];        /* Pipe file descriptors. */
+  long prebuffer;    /* Number of chunks to prebuffer before starting the writer */
+  devices_t *devices;/* Device to use */
   chunk_t *reader;   /* Chunk the reader is busy with */
   chunk_t *writer;   /* Chunk the writer is busy with */
   chunk_t *end;      /* Last chunk in the buffer (for convenience) */
   chunk_t buffer[1]; /* The buffer itself. It's more than one chunk. */
 } buf_t;
 
-buf_t *fork_writer (long size, devices_t *d);
+buf_t *create_buffer (long size, devices_t *d, long prebuffer);
+void fork_writer (buf_t * buf);
 void submit_chunk (buf_t *buf, chunk_t chunk);
 void buffer_shutdown (buf_t *buf);
 
 #define STAT_FLUSH 1
 #define STAT_SHUTDOWN 2
+#define STAT_PLAYING 4
 
 
 #endif /* !defined (__BUFFER_H) */
Index: ogg123.h
===================================================================
RCS file: /usr/local/cvsroot/vorbis-tools/ogg123/ogg123.h,v
retrieving revision 1.6
diff -u -r1.6 ogg123.h
--- ogg123.h	2001/02/22 02:49:12	1.6
+++ ogg123.h	2001/03/03 21:03:55
@@ -30,6 +30,7 @@
   FILE *instream;             /* Stream to read from. */
   devices_t *outdevices;      /* Streams to write to. */
   int buffer_size;            /* Size of the buffer in chunks. */
+  long prebuffer;             /* Number of chunks to prebuffer. */
   int rate, channels;         /* playback params for opening audio devices */
 } ogg123_options_t;           /* Changed in 0.6 to be non-static */
 
Index: buffer.c
===================================================================
RCS file: /usr/local/cvsroot/vorbis-tools/ogg123/buffer.c,v
retrieving revision 1.3
diff -u -r1.3 buffer.c
--- buffer.c	2001/01/30 10:42:48	1.3
+++ buffer.c	2001/03/03 21:03:55
@@ -29,9 +29,10 @@
 }
 
 /* Main write loop. No semaphores. No deadlock. No problem. I hope. */
-void writer_main (volatile buf_t *buf, devices_t *d)
+void writer_main (volatile buf_t *buf)
 {
   devices_t *d1;
+
   while (! (buf->status & STAT_SHUTDOWN && buf->reader == buf->writer))
     {
       /* Writer just waits on reader to be done with buf_write.
@@ -46,7 +47,7 @@
 
       /* devices_write (buf->writer->data, buf->writer->len, d); */
       {
-	d1 = d;
+	d1 = buf->devices;
 	while (d1 != NULL) {
 	  ao_play(d1->device, buf->writer->data, buf->writer->len);
 	  d1 = d1->next_device;
@@ -68,9 +69,8 @@
  * to the buffer structure that is shared. Just pass this straight to
  * submit_chunk and all will be happy. */
 
-buf_t *fork_writer (long size, devices_t *d)
+buf_t *create_buffer (long size, devices_t *d, long prebuffer)
 {
-  int childpid;
   buf_t *buf;
 
   /* Get the shared memory segment. */
@@ -94,6 +94,8 @@
     }
   
   buffer_init (buf, size);
+  buf->devices = d;
+  buf->prebuffer = prebuffer;
   
   /* Create a pipe for communication between the two processes. Unlike
    * the first incarnation of an ogg123 buffer, the data is not transferred
@@ -110,6 +112,18 @@
 
   fflush (stdout);
   /* buffer flushes stderr, but stderr is unbuffered (*duh*!) */
+
+  if(prebuffer == -1)
+	  fork_writer (buf);
+  
+  return buf;
+}
+
+void fork_writer(buf_t * buf)
+{
+  int childpid;
+
+  buf->status |= STAT_PLAYING;
   
   childpid = fork();
   
@@ -121,11 +135,9 @@
 
   if (childpid == 0)
     {
-      writer_main (buf, d);
-      return NULL;
+      writer_main (buf);
+      exit(1);
     }
-  else
-    return buf;
 }
 
 void submit_chunk (buf_t *buf, chunk_t chunk)
@@ -143,7 +155,7 @@
       /* buffer overflow (yikes! no actually it's a GOOD thing) */
       int ret;
       char t;
-      
+  
       tv.tv_sec = 1;
       tv.tv_usec = 0;
       ret = select (buf->fds[0]+1, &set, NULL, NULL, &tv);
@@ -151,13 +163,18 @@
       while (ret-- > 0)
 	read (buf->fds[0], &t, 1);
     }
-	      
+
   *(buf->reader) = chunk;
   /* do this atomically */
   if (buf->reader == buf->end)
     buf->reader = buf->buffer;
   else
     buf->reader++;
+
+  /* If we've buffered enough and the writer hasn't been started yet, start it */
+  if (! (buf->status & STAT_PLAYING) )
+    if (buf->reader - buf->buffer == buf->prebuffer)
+      fork_writer (buf);
 }
 
 void buffer_shutdown (buf_t *buf)
@@ -165,6 +182,13 @@
   struct timeval tv;
 
   buf->status |= STAT_SHUTDOWN;
+
+  if (!(buf->status & STAT_PLAYING))
+  {
+    /* Start the writer process */
+    fork_writer (buf);
+  }
+  
   while (buf->status != 0)
     {
       tv.tv_sec = 1;
Index: ogg123.c
===================================================================
RCS file: /usr/local/cvsroot/vorbis-tools/ogg123/ogg123.c,v
retrieving revision 1.30
diff -u -r1.30 ogg123.c
--- ogg123.c	2001/02/24 00:52:22	1.30
+++ ogg123.c	2001/03/03 21:03:55
@@ -64,6 +64,7 @@
     {"quiet", no_argument, 0, 'q'},
     {"shuffle", no_argument, 0, 'z'},
     {"buffer", required_argument, 0, 'b'},
+    {"prebuffer", required_argument, 0, 'p'},
     {0, 0, 0, 0}
 };
 
@@ -87,6 +88,7 @@
 	    "      v to previously specified device (with -d).  See\n"
 	    "      man page for more info.\n"
 	    "  -b n, --buffer n  use a buffer of 'n' chunks (4096 bytes)\n"
+	    "  -p n, --prebuffer n  decode 'n' chunks before starting to play\n"
 	    "  -v, --verbose  display progress and other useful stuff\n"
 	    "  -q, --quiet    don't display anything (no title)\n"
 	    "  -z, --shuffle  shuffle play\n");
@@ -110,10 +112,11 @@
     opt.instream = NULL;
     opt.outdevices = NULL;
     opt.buffer_size = 0;
+    opt.prebuffer = -1;
 
     ao_initialize();
 
-    while (-1 != (ret = getopt_long(argc, argv, "b:d:hk:o:qvVz",
+    while (-1 != (ret = getopt_long(argc, argv, "b:p:d:hk:o:qvVz",
 				    long_options, &option_index))) {
 	switch (ret) {
 	case 0:
@@ -123,6 +126,14 @@
 	case 'b':
 	  opt.buffer_size = atoi (optarg);
 	  break;
+	case 'p':
+	  opt.prebuffer = atoi (optarg) - 1;
+	  if (opt.prebuffer >= opt.buffer_size)
+	  {
+		  fprintf(stderr, "Prebuffer must be less than or equal to the buffer size\n");
+		  exit(1);
+	  }
+	  break;
 	case 'd':
 	    temp_driver_id = ao_get_driver_id(optarg);
 	    if (temp_driver_id < 0) {
@@ -212,6 +223,9 @@
 	}
     }
 
+    if (buffer != NULL)
+            buffer_shutdown(buffer);
+
     while (opt.outdevices != NULL) {
       ao_close(opt.outdevices->device);
       current = opt.outdevices->next_device;
@@ -219,9 +233,6 @@
       opt.outdevices = current;
     }
 
-    if (buffer != NULL)
-	    buffer_shutdown(buffer);
-    
     ao_shutdown();
 
     return (0);
@@ -506,7 +517,7 @@
   }
   
   if (opt->buffer_size)
-    *buffer = fork_writer (opt->buffer_size, opt->outdevices);
+    *buffer = create_buffer (opt->buffer_size, opt->outdevices, opt->prebuffer);
   
     return 0;
 }
